@@ -15,9 +15,13 @@
  */
 package org.gradle.groovy.scripts.internal;
 
+import groovy.lang.Script;
+import org.codehaus.groovy.ast.ClassNode;
+import org.gradle.api.Action;
+import org.gradle.api.internal.initialization.loadercache.ClassLoaderId;
 import org.gradle.groovy.scripts.ScriptSource;
-import org.gradle.groovy.scripts.Transformer;
 
 public interface ScriptClassCompiler {
-    <T extends groovy.lang.Script> Class<? extends T> compile(ScriptSource source, ClassLoader classLoader, Transformer transformer, Class<T> scriptBaseClass);
+    <T extends Script, M> CompiledScript<T, M> compile(ScriptSource source, ClassLoader classLoader, ClassLoaderId classLoaderId, CompileOperation<M> transformer, String classpathClosureName, Class<T> scriptBaseClass,
+                                                       Action<? super ClassNode> verifier);
 }
