@@ -16,12 +16,15 @@
 package org.gradle.logging.internal;
 
 import org.gradle.api.logging.LogLevel;
+import org.gradle.internal.progress.OperationIdentifier;
 
 public class ProgressEvent extends CategorisedOutputEvent {
     private final String status;
+    private OperationIdentifier operationId;
 
-    public ProgressEvent(long timestamp, String category, String status) {
+    public ProgressEvent(OperationIdentifier operationId, long timestamp, String category, String status) {
         super(timestamp, category, LogLevel.LIFECYCLE);
+        this.operationId = operationId;
         this.status = status;
     }
 
@@ -33,4 +36,8 @@ public class ProgressEvent extends CategorisedOutputEvent {
     public String toString() {
         return String.format("Progress %s", status);
     }
+
+    public OperationIdentifier getOperationId() {
+        return operationId;
+    }
 }
