@@ -15,12 +15,16 @@
  */
 package org.gradle.integtests.fixtures.executer;
 
+import com.google.common.collect.ImmutableList;
+import com.google.common.collect.Sets;
 import groovy.lang.Closure;
 import org.gradle.api.Action;
 import org.gradle.api.internal.ClosureBackedAction;
+import org.gradle.api.internal.initialization.DefaultClassLoaderScope;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
 import org.gradle.internal.jvm.Jvm;
+import org.gradle.launcher.daemon.configuration.DaemonParameters;
 import org.gradle.launcher.daemon.configuration.GradleProperties;
 import org.gradle.listener.ActionBroadcast;
 import org.gradle.process.internal.JvmOptions;
@@ -36,13 +40,26 @@ import java.nio.charset.Charset;
 import java.util.*;
 
 import static java.util.Arrays.asList;
-import static org.gradle.util.Matchers.*;
+import static org.gradle.util.Matchers.containsLine;
+import static org.gradle.util.Matchers.matchesRegexp;
 
 public abstract class AbstractGradleExecuter implements GradleExecuter {
+    protected static Set<String> propagatedSystemProperties = Sets.newHashSet();
+
+    public static void propagateSystemProperty(String name) {
+        propagatedSystemProperties.add(name);
+    }
+
+    private static final String DEBUG_SYSPROP = "org.gradle.integtest.debug";
+
+    protected static final List<String> DEBUG_ARGS = ImmutableList.of(
+        "-Xdebug",
+        "-Xrunjdwp:transport=dt_socket,server=y,suspend=y,address=5005"
+    );
 
     private final Logger logger;
 
-    private final IntegrationTestBuildContext buildContext = new IntegrationTestBuildContext();
+    protected final IntegrationTestBuildContext buildContext = new IntegrationTestBuildContext();
 
     private final List<String> args = new ArrayList<String>();
     private final List<String> tasks = new ArrayList<String>();
@@ -63,21 +80,28 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     private File settingsFile;
     private InputStream stdin;
     private String defaultCharacterEncoding;
-    private Integer daemonIdleTimeoutSecs = 10;
+    private String tmpDir;
+    private Locale defaultLocale;
+    private int daemonIdleTimeoutSecs = 60;
     private File daemonBaseDir = buildContext.getDaemonBaseDir();
     private final List<String> gradleOpts = new ArrayList<String>();
     private boolean noDefaultJvmArgs;
     private boolean requireGradleHome;
+    private boolean daemonStartingMessageDisabled = true;
 
     private boolean deprecationChecksOn = true;
+    private boolean eagerClassLoaderCreationChecksOn = true;
     private boolean stackTraceChecksOn = true;
 
     private final ActionBroadcast<GradleExecuter> beforeExecute = new ActionBroadcast<GradleExecuter>();
-    private final Set<Action> afterExecute = new LinkedHashSet<Action>();
+    private final Set<Action<? super GradleExecuter>> afterExecute = new LinkedHashSet<Action<? super GradleExecuter>>();
 
     private final TestDirectoryProvider testDirectoryProvider;
     private final GradleDistribution distribution;
 
+    private boolean debug = Boolean.getBoolean(DEBUG_SYSPROP);
+    protected boolean interactive;
+
     protected AbstractGradleExecuter(GradleDistribution distribution, TestDirectoryProvider testDirectoryProvider) {
         this.distribution = distribution;
         this.testDirectoryProvider = testDirectoryProvider;
@@ -105,9 +129,13 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         environmentVars.clear();
         stdin = null;
         defaultCharacterEncoding = null;
+        tmpDir = null;
+        defaultLocale = null;
         noDefaultJvmArgs = false;
         deprecationChecksOn = true;
         stackTraceChecksOn = true;
+        debug = Boolean.getBoolean(DEBUG_SYSPROP);
+        interactive = false;
         return this;
     }
 
@@ -191,6 +219,12 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (defaultCharacterEncoding != null) {
             executer.withDefaultCharacterEncoding(defaultCharacterEncoding);
         }
+        if (tmpDir != null) {
+            executer.withTmpDir(tmpDir);
+        }
+        if (defaultLocale != null) {
+            executer.withDefaultLocale(defaultLocale);
+        }
         executer.withGradleOpts(gradleOpts.toArray(new String[gradleOpts.size()]));
         if (noDefaultJvmArgs) {
             executer.withNoDefaultJvmArgs();
@@ -200,13 +234,21 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (!deprecationChecksOn) {
             executer.withDeprecationChecksDisabled();
         }
+        if (!eagerClassLoaderCreationChecksOn) {
+            executer.withEagerClassLoaderCreationCheckDisabled();
+        }
         if (!stackTraceChecksOn) {
             executer.withStackTraceChecksDisabled();
         }
         if (requireGradleHome) {
             executer.requireGradleHome();
         }
+        if (!daemonStartingMessageDisabled) {
+            executer.withDaemonStartingMessageEnabled();
+        }
 
+        executer.withDebug(debug);
+        executer.withForceInteractive(interactive);
         return executer;
     }
 
@@ -251,6 +293,18 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
      * Returns the gradle opts set with withGradleOpts() (does not consider any set via withEnvironmentVars())
      */
     protected List<String> getGradleOpts() {
+        List<String> gradleOpts = new ArrayList<String>(this.gradleOpts);
+        for (Map.Entry<String, String> entry : getImplicitJvmSystemProperties().entrySet()) {
+            String key = entry.getKey();
+            String value = entry.getValue();
+            gradleOpts.add(String.format("-D%s=%s", key, value));
+        }
+        gradleOpts.add("-ea");
+
+        if (isDebug()) {
+            gradleOpts.addAll(DEBUG_ARGS);
+        }
+
         return gradleOpts;
     }
 
@@ -296,10 +350,24 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return this;
     }
 
+    public GradleExecuter withTmpDir(String tmpDir) {
+        this.tmpDir = tmpDir;
+        return this;
+    }
+
     public String getDefaultCharacterEncoding() {
         return defaultCharacterEncoding == null ? Charset.defaultCharset().name() : defaultCharacterEncoding;
     }
 
+    public GradleExecuter withDefaultLocale(Locale defaultLocale) {
+        this.defaultLocale = defaultLocale;
+        return this;
+    }
+
+    public Locale getDefaultLocale() {
+        return defaultLocale;
+    }
+
     public GradleExecuter withSearchUpwards() {
         searchUpwards = true;
         return this;
@@ -368,7 +436,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
                 result.append(" ");
             }
             if (jvmArg.contains(" ")) {
-                assert !jvmArg.contains("\"");
+                assert !jvmArg.contains("\"") : "jvmArg '" + jvmArg + "' contains '\"'";
                 result.append('"');
                 result.append(jvmArg);
                 result.append('"');
@@ -498,15 +566,32 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         }
 
         properties.put(GradleProperties.IDLE_TIMEOUT_PROPERTY, "" + (daemonIdleTimeoutSecs * 1000));
-        properties.put(GradleProperties.BASE_DIR_PROPERTY, daemonBaseDir.getAbsolutePath());
+        properties.put(GradleProperties.DAEMON_BASE_DIR_PROPERTY, daemonBaseDir.getAbsolutePath());
         properties.put(DeprecationLogger.ORG_GRADLE_DEPRECATION_TRACE_PROPERTY_NAME, "true");
 
-        String tmpDirPath = getTmpDir().createDir().getAbsolutePath();
+        String tmpDirPath = tmpDir;
+        if (tmpDirPath == null) {
+            tmpDirPath = getDefaultTmpDir().createDir().getAbsolutePath();
+        }
         if (!tmpDirPath.contains(" ") || getDistribution().isSupportsSpacesInGradleAndJavaOpts()) {
             properties.put("java.io.tmpdir", tmpDirPath);
         }
 
         properties.put("file.encoding", getDefaultCharacterEncoding());
+        Locale locale = getDefaultLocale();
+        if (locale != null) {
+            properties.put("user.language", locale.getLanguage());
+            properties.put("user.country", locale.getCountry());
+            properties.put("user.variant", locale.getVariant());
+        }
+
+        if (eagerClassLoaderCreationChecksOn) {
+            properties.put(DefaultClassLoaderScope.STRICT_MODE_PROPERTY, "true");
+        }
+
+        if (interactive) {
+            properties.put(DaemonParameters.INTERACTIVE_TOGGLE, "true");
+        }
 
         return properties;
     }
@@ -624,12 +709,17 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return this;
     }
 
+    public GradleExecuter withEagerClassLoaderCreationCheckDisabled() {
+        eagerClassLoaderCreationChecksOn = false;
+        return this;
+    }
+
     public GradleExecuter withStackTraceChecksDisabled() {
         stackTraceChecksOn = false;
         return this;
     }
 
-    protected TestFile getTmpDir() {
+    protected TestFile getDefaultTmpDir() {
         return new TestFile(getTestDirectoryProvider().getTestDirectory(), "tmp");
     }
 
@@ -650,4 +740,30 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         this.requireGradleHome = true;
         return this;
     }
+
+    public GradleExecuter withDaemonStartingMessageEnabled() {
+        daemonStartingMessageDisabled = false;
+        return this;
+    }
+
+    public boolean isDaemonStartingMessageDisabled() {
+        return daemonStartingMessageDisabled;
+    }
+
+    @Override
+    public GradleExecuter withDebug(boolean flag) {
+        debug = flag;
+        return this;
+    }
+
+    @Override
+    public GradleExecuter withForceInteractive(boolean flag) {
+        interactive = flag;
+        return this;
+    }
+
+    @Override
+    public boolean isDebug() {
+        return debug;
+    }
 }
