@@ -16,37 +16,46 @@
 
 package org.gradle.integtests.fixtures.executer;
 
-import org.gradle.BuildListener;
 import org.gradle.BuildResult;
-import org.gradle.GradleLauncher;
 import org.gradle.StartParameter;
 import org.gradle.api.GradleException;
 import org.gradle.api.Task;
 import org.gradle.api.execution.TaskExecutionGraph;
 import org.gradle.api.execution.TaskExecutionGraphListener;
 import org.gradle.api.execution.TaskExecutionListener;
-import org.gradle.api.initialization.Settings;
-import org.gradle.api.internal.LocationAwareException;
 import org.gradle.api.internal.classpath.DefaultModuleRegistry;
-import org.gradle.api.internal.file.IdentityFileResolver;
-import org.gradle.api.invocation.Gradle;
-import org.gradle.api.logging.LogLevel;
+import org.gradle.api.internal.file.TestFiles;
 import org.gradle.api.logging.StandardOutputListener;
 import org.gradle.api.tasks.TaskState;
 import org.gradle.cli.CommandLineParser;
-import org.gradle.cli.ParsedCommandLine;
+import org.gradle.configuration.GradleLauncherMetaData;
 import org.gradle.execution.MultipleBuildFailures;
-import org.gradle.initialization.DefaultCommandLineConverter;
-import org.gradle.initialization.DefaultGradleLauncherFactory;
+import org.gradle.initialization.*;
 import org.gradle.internal.Factory;
+import org.gradle.internal.SystemProperties;
+import org.gradle.internal.event.ListenerManager;
+import org.gradle.internal.exceptions.LocationAwareException;
+import org.gradle.internal.invocation.BuildAction;
 import org.gradle.internal.jvm.Jvm;
-import org.gradle.internal.nativeplatform.ProcessEnvironment;
-import org.gradle.internal.nativeplatform.services.NativeServices;
+import org.gradle.internal.nativeintegration.ProcessEnvironment;
+import org.gradle.internal.service.ServiceRegistry;
+import org.gradle.internal.service.ServiceRegistryBuilder;
+import org.gradle.internal.service.scopes.GlobalScopeServices;
 import org.gradle.launcher.Main;
-import org.gradle.launcher.daemon.configuration.GradlePropertiesConfigurer;
-import org.gradle.launcher.daemon.registry.DaemonRegistry;
+import org.gradle.launcher.cli.ExecuteBuildAction;
+import org.gradle.launcher.cli.Parameters;
+import org.gradle.launcher.cli.ParametersConverter;
+import org.gradle.launcher.daemon.configuration.DaemonUsage;
+import org.gradle.launcher.exec.BuildActionExecuter;
+import org.gradle.launcher.exec.BuildActionParameters;
+import org.gradle.launcher.exec.DefaultBuildActionParameters;
+import org.gradle.initialization.ReportedException;
+import org.gradle.logging.LoggingServiceRegistry;
+import org.gradle.logging.ShowStacktrace;
 import org.gradle.process.internal.JavaExecHandleBuilder;
 import org.gradle.test.fixtures.file.TestDirectoryProvider;
+import org.gradle.test.fixtures.file.TestFile;
+import org.gradle.testfixtures.internal.NativeServicesTestFixture;
 import org.gradle.util.DeprecationLogger;
 import org.hamcrest.Matcher;
 import org.hamcrest.Matchers;
@@ -54,18 +63,27 @@ import org.hamcrest.Matchers;
 import java.io.File;
 import java.io.InputStream;
 import java.io.StringWriter;
+import java.lang.management.ManagementFactory;
 import java.nio.charset.Charset;
 import java.util.*;
 import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.CopyOnWriteArraySet;
+import java.util.regex.Pattern;
 
 import static org.gradle.util.Matchers.*;
 import static org.hamcrest.Matchers.*;
 import static org.junit.Assert.*;
 
 class InProcessGradleExecuter extends AbstractGradleExecuter {
+    private static final ServiceRegistry GLOBAL_SERVICES = ServiceRegistryBuilder.builder()
+        .displayName("Global services")
+        .parent(LoggingServiceRegistry.newCommandLineProcessLogging())
+        .parent(NativeServicesTestFixture.getInstance())
+        .provider(new GlobalScopeServices(true))
+        .build();
+    private final ProcessEnvironment processEnvironment = GLOBAL_SERVICES.get(ProcessEnvironment.class);
 
-    private final ProcessEnvironment processEnvironment = NativeServices.getInstance().get(ProcessEnvironment.class);
+    public static final TestFile COMMON_TMP = new TestFile(new File("build/tmp"));
 
     InProcessGradleExecuter(GradleDistribution distribution, TestDirectoryProvider testDirectoryProvider) {
         super(distribution, testDirectoryProvider);
@@ -79,8 +97,8 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
 
     @Override
     protected ExecutionResult doRun() {
-        OutputListenerImpl outputListener = new OutputListenerImpl();
-        OutputListenerImpl errorListener = new OutputListenerImpl();
+        StandardOutputListener outputListener = new OutputListenerImpl();
+        StandardOutputListener errorListener = new OutputListenerImpl();
         BuildListenerImpl buildListener = new BuildListenerImpl();
         BuildResult result = doRun(outputListener, errorListener, buildListener);
         try {
@@ -89,20 +107,20 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
             throw new UnexpectedBuildFailure(e);
         }
         return assertResult(new InProcessExecutionResult(buildListener.executedTasks, buildListener.skippedTasks,
-                outputListener.toString(), errorListener.toString()));
+            new OutputScrapingExecutionResult(outputListener.toString(), errorListener.toString())));
     }
 
     @Override
     protected ExecutionFailure doRunWithFailure() {
-        OutputListenerImpl outputListener = new OutputListenerImpl();
-        OutputListenerImpl errorListener = new OutputListenerImpl();
+        StandardOutputListener outputListener = new OutputListenerImpl();
+        StandardOutputListener errorListener = new OutputListenerImpl();
         BuildListenerImpl buildListener = new BuildListenerImpl();
         try {
             doRun(outputListener, errorListener, buildListener).rethrowFailure();
             throw new AssertionError("expected build to fail but it did not.");
         } catch (GradleException e) {
             return assertResult(new InProcessExecutionFailure(buildListener.executedTasks, buildListener.skippedTasks,
-                    outputListener.writer.toString(), errorListener.writer.toString(), e));
+                new OutputScrapingExecutionFailure(outputListener.toString(), errorListener.toString()), e));
         }
     }
 
@@ -115,18 +133,23 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
     protected GradleHandle doStart() {
         return new ForkingGradleHandle(getResultAssertion(), getDefaultCharacterEncoding(), new Factory<JavaExecHandleBuilder>() {
             public JavaExecHandleBuilder create() {
-                JavaExecHandleBuilder builder = new JavaExecHandleBuilder(new IdentityFileResolver());
+                JavaExecHandleBuilder builder = new JavaExecHandleBuilder(TestFiles.resolver());
                 builder.workingDir(getWorkingDir());
                 Set<File> classpath = new DefaultModuleRegistry().getFullClasspath();
                 builder.classpath(classpath);
+                builder.jvmArgs(getGradleOpts());
                 builder.setMain(Main.class.getName());
                 builder.args(getAllArgs());
+                builder.setStandardInput(getStdin());
+                if (isDebug()) {
+                    builder.jvmArgs(DEBUG_ARGS);
+                }
                 return builder;
             }
         }).start();
     }
 
-    private BuildResult doRun(final OutputListenerImpl outputListener, OutputListenerImpl errorListener, BuildListenerImpl listener) {
+    private BuildResult doRun(StandardOutputListener outputListener, StandardOutputListener errorListener, BuildListenerImpl listener) {
         // Capture the current state of things that we will change during execution
         InputStream originalStdIn = System.in;
         Properties originalSysProperties = new Properties();
@@ -143,32 +166,48 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
         Map<String, String> implicitJvmSystemProperties = getImplicitJvmSystemProperties();
         System.getProperties().putAll(implicitJvmSystemProperties);
 
-        StartParameter parameter = new StartParameter();
-        parameter.setLogLevel(LogLevel.INFO);
-        parameter.setSearchUpwards(true);
-        parameter.setCurrentDir(getWorkingDir());
+        // TODO: Fix tests that rely on this being set before we process arguments like this...
+        StartParameter startParameter = new StartParameter();
+        startParameter.setCurrentDir(getWorkingDir());
+        startParameter.setShowStacktrace(ShowStacktrace.ALWAYS);
 
         CommandLineParser parser = new CommandLineParser();
-        DefaultCommandLineConverter converter = new DefaultCommandLineConverter();
-        converter.configure(parser);
-        ParsedCommandLine parsedCommandLine = parser.parse(getAllArgs());
-        converter.convert(parsedCommandLine, parameter);
+        ParametersConverter parametersConverter = new ParametersConverter();
+        parametersConverter.configure(parser);
+        parametersConverter.convert(parser.parse(getAllArgs()), new Parameters(startParameter));
 
-        //I'm not sure if below is safe
-        new GradlePropertiesConfigurer().configureStartParameter(parameter);
+        BuildActionExecuter<BuildActionParameters> actionExecuter = GLOBAL_SERVICES.get(BuildActionExecuter.class);
+
+        ListenerManager listenerManager = GLOBAL_SERVICES.get(ListenerManager.class);
+        listenerManager.addListener(listener);
 
-        DefaultGradleLauncherFactory factory = (DefaultGradleLauncherFactory) GradleLauncher.getFactory();
-        factory.addListener(listener);
-        GradleLauncher gradleLauncher = GradleLauncher.newInstance(parameter);
-        gradleLauncher.addStandardOutputListener(outputListener);
-        gradleLauncher.addStandardErrorListener(errorListener);
         try {
-            return gradleLauncher.run();
+            // TODO: Reuse more of BuildActionsFactory
+            BuildAction action = new ExecuteBuildAction(startParameter);
+            BuildActionParameters buildActionParameters = new DefaultBuildActionParameters(
+                System.getProperties(),
+                System.getenv(),
+                SystemProperties.getInstance().getCurrentDir(),
+                startParameter.getLogLevel(),
+                DaemonUsage.EXPLICITLY_DISABLED,
+                startParameter.isContinuous(),
+                interactive
+            );
+            BuildRequestContext buildRequestContext = new DefaultBuildRequestContext(
+                new DefaultBuildRequestMetaData(new GradleLauncherMetaData(),
+                    ManagementFactory.getRuntimeMXBean().getStartTime()),
+                new DefaultBuildCancellationToken(),
+                new NoOpBuildEventConsumer(),
+                outputListener, errorListener);
+            actionExecuter.execute(action, buildRequestContext, buildActionParameters);
+            return new BuildResult(null, null);
+        } catch (ReportedException e) {
+            return new BuildResult(null, e.getCause());
         } finally {
             // Restore the environment
             System.setProperties(originalSysProperties);
             processEnvironment.maybeSetProcessDir(originalUserDir);
-            for (String envVar: getEnvironmentVars().keySet()) {
+            for (String envVar : getEnvironmentVars().keySet()) {
                 String oldValue = originalEnv.get(envVar);
                 if (oldValue != null) {
                     processEnvironment.maybeSetEnvironmentVariable(envVar, oldValue);
@@ -176,15 +215,11 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
                     processEnvironment.maybeRemoveEnvironmentVariable(envVar);
                 }
             }
-            factory.removeListener(listener);
+            listenerManager.removeListener(listener);
             System.setIn(originalStdIn);
         }
     }
 
-    public DaemonRegistry getDaemonRegistry() {
-        throw new UnsupportedOperationException();
-    }
-
     public void assertCanExecute() {
         assertNull(getExecutable());
         assertEquals(getJavaHome(), Jvm.current().getJavaHome());
@@ -192,10 +227,23 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
         if (defaultEncoding != null) {
             assertEquals(Charset.forName(defaultEncoding), Charset.defaultCharset());
         }
+        Locale defaultLocale = getDefaultLocale();
+        if (defaultLocale != null) {
+            assertEquals(defaultLocale, Locale.getDefault());
+        }
         assertFalse(isRequireGradleHome());
     }
 
-    private static class BuildListenerImpl implements TaskExecutionGraphListener, BuildListener {
+    @Override
+    protected TestFile getDefaultTmpDir() {
+        // File.createTempFile sets the location of the temp directory to a static variable on the first call.  This prevents future
+        // changes to java.io.tmpdir from having any effect in the same process.  We set this to use a common tmp directory for all
+        // tests running in the same process so that we don't have a situation where one process initializes with a tmp directory
+        // that it then removes, causing an IOException for any future tests that run in the same process and call File.createTempFile.
+        return COMMON_TMP;
+    }
+
+    private static class BuildListenerImpl implements TaskExecutionGraphListener {
         private final List<String> executedTasks = new CopyOnWriteArrayList<String>();
         private final Set<String> skippedTasks = new CopyOnWriteArraySet<String>();
 
@@ -203,26 +251,6 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
             List<Task> planned = new ArrayList<Task>(graph.getAllTasks());
             graph.addTaskExecutionListener(new TaskListenerImpl(planned, executedTasks, skippedTasks));
         }
-
-        public void buildStarted(Gradle gradle) {
-            DeprecationLogger.setLogTrace(true);
-        }
-
-        public void settingsEvaluated(Settings settings) {
-
-        }
-
-        public void projectsLoaded(Gradle gradle) {
-
-        }
-
-        public void projectsEvaluated(Gradle gradle) {
-
-        }
-
-        public void buildFinished(BuildResult result) {
-            DeprecationLogger.setLogTrace(false);
-        }
     }
 
     private static class OutputListenerImpl implements StandardOutputListener {
@@ -279,28 +307,31 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
     public static class InProcessExecutionResult implements ExecutionResult {
         private final List<String> plannedTasks;
         private final Set<String> skippedTasks;
-        private final String output;
-        private final String error;
+        private final OutputScrapingExecutionResult outputResult;
 
-        public InProcessExecutionResult(List<String> plannedTasks, Set<String> skippedTasks, String output,
-                                        String error) {
+        public InProcessExecutionResult(List<String> plannedTasks, Set<String> skippedTasks, OutputScrapingExecutionResult outputResult) {
             this.plannedTasks = plannedTasks;
             this.skippedTasks = skippedTasks;
-            this.output = output;
-            this.error = error;
+            this.outputResult = outputResult;
         }
 
         public String getOutput() {
-            return output;
+            return outputResult.getOutput();
         }
 
-        public ExecutionResult assertOutputEquals(String expectedOutput, boolean ignoreExtraLines) {
-            new SequentialOutputMatcher().assertOutputMatches(expectedOutput, getOutput(), ignoreExtraLines);
+        public ExecutionResult assertOutputEquals(String expectedOutput, boolean ignoreExtraLines, boolean ignoreLineOrder) {
+            outputResult.assertOutputEquals(expectedOutput, ignoreExtraLines, ignoreLineOrder);
+            return this;
+        }
+
+        @Override
+        public ExecutionResult assertOutputContains(String expectedOutput) {
+            outputResult.assertOutputContains(expectedOutput);
             return this;
         }
 
         public String getError() {
-            return error;
+            return outputResult.getError();
         }
 
         public List<String> getExecutedTasks() {
@@ -310,6 +341,7 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
         public ExecutionResult assertTasksExecuted(String... taskPaths) {
             List<String> expected = Arrays.asList(taskPaths);
             assertThat(plannedTasks, equalTo(expected));
+            outputResult.assertTasksExecuted(taskPaths);
             return this;
         }
 
@@ -318,25 +350,41 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
         }
 
         public ExecutionResult assertTasksSkipped(String... taskPaths) {
+            if (GradleContextualExecuter.isParallel()) {
+                return this;
+            }
             Set<String> expected = new HashSet<String>(Arrays.asList(taskPaths));
             assertThat(skippedTasks, equalTo(expected));
+            outputResult.assertTasksSkipped(taskPaths);
             return this;
         }
 
         public ExecutionResult assertTaskSkipped(String taskPath) {
+            if (GradleContextualExecuter.isParallel()) {
+                return this;
+            }
             assertThat(skippedTasks, hasItem(taskPath));
+            outputResult.assertTaskSkipped(taskPath);
             return this;
         }
 
         public ExecutionResult assertTasksNotSkipped(String... taskPaths) {
+            if (GradleContextualExecuter.isParallel()) {
+                return this;
+            }
             Set<String> expected = new HashSet<String>(Arrays.asList(taskPaths));
             Set<String> notSkipped = getNotSkippedTasks();
             assertThat(notSkipped, equalTo(expected));
+            outputResult.assertTasksNotSkipped(taskPaths);
             return this;
         }
 
         public ExecutionResult assertTaskNotSkipped(String taskPath) {
+            if (GradleContextualExecuter.isParallel()) {
+                return this;
+            }
             assertThat(getNotSkippedTasks(), hasItem(taskPath));
+            outputResult.assertTaskNotSkipped(taskPath);
             return this;
         }
 
@@ -348,27 +396,51 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
     }
 
     private static class InProcessExecutionFailure extends InProcessExecutionResult implements ExecutionFailure {
+        private static final Pattern LOCATION_PATTERN = Pattern.compile("(?m)^((\\w+ )+'.+') line: (\\d+)$");
+        private final OutputScrapingExecutionFailure outputFailure;
         private final GradleException failure;
+        private final String fileName;
+        private final String lineNumber;
+        private final String description;
 
-        public InProcessExecutionFailure(List<String> tasks, Set<String> skippedTasks, String output, String error,
+        public InProcessExecutionFailure(List<String> tasks, Set<String> skippedTasks, OutputScrapingExecutionFailure outputFailure,
                                          GradleException failure) {
-            super(tasks, skippedTasks, output, error);
+            super(tasks, skippedTasks, outputFailure);
+            this.outputFailure = outputFailure;
             this.failure = failure;
+
+            // Chop up the exception message into its expected parts
+            java.util.regex.Matcher matcher = LOCATION_PATTERN.matcher(failure.getMessage());
+            if (matcher.find()) {
+                fileName = matcher.group(1);
+                lineNumber = matcher.group(3);
+                description = failure.getMessage().substring(matcher.end()).trim();
+            } else {
+                fileName = "";
+                lineNumber = "";
+                description = failure.getMessage().trim();
+            }
         }
 
         public ExecutionFailure assertHasLineNumber(int lineNumber) {
-            assertThat(failure.getMessage(), containsString(String.format(" line: %d", lineNumber)));
+            assertThat(this.lineNumber, equalTo(String.valueOf(lineNumber)));
+            outputFailure.assertHasLineNumber(lineNumber);
             return this;
-
         }
 
         public ExecutionFailure assertHasFileName(String filename) {
-            assertThat(failure.getMessage(), startsWith(String.format("%s", filename)));
+            assertThat(this.fileName, equalTo(filename));
+            outputFailure.assertHasFileName(filename);
+            return this;
+        }
+
+        public ExecutionFailure assertHasResolution(String resolution) {
+            outputFailure.assertHasResolution(resolution);
             return this;
         }
 
         public ExecutionFailure assertHasCause(String description) {
-            assertThatCause(startsWith(description));
+            assertThatCause(normalizedLineSeparators(startsWith(description)));
             return this;
         }
 
@@ -376,6 +448,7 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
             List<Throwable> causes = new ArrayList<Throwable>();
             extractCauses(failure, causes);
             assertThat(causes, Matchers.<Throwable>hasItem(hasMessage(matcher)));
+            outputFailure.assertThatCause(matcher);
             return this;
         }
 
@@ -388,7 +461,7 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
             } else if (failure instanceof LocationAwareException) {
                 causes.addAll(((LocationAwareException) failure).getReportableCauses());
             } else {
-                causes.add(failure.getCause());
+                causes.add(failure);
             }
         }
 
@@ -399,6 +472,7 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
             } else {
                 assertThat(failure.getCause(), nullValue());
             }
+            outputFailure.assertHasNoCause();
             return this;
         }
 
@@ -408,7 +482,8 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
         }
 
         public ExecutionFailure assertThatDescription(Matcher<String> matcher) {
-            assertThat(failure.getMessage(), containsLine(matcher));
+            assertThat(description, normalizedLineSeparators(matcher));
+            outputFailure.assertThatDescription(matcher);
             return this;
         }
 
@@ -417,8 +492,8 @@ class InProcessGradleExecuter extends AbstractGradleExecuter {
             return this;
         }
 
-        public DependencyResolutionFailure assertResolutionFailure(String configuration) {
-            return new DependencyResolutionFailure(this, configuration);
+        public DependencyResolutionFailure assertResolutionFailure(String configurationPath) {
+            return new DependencyResolutionFailure(this, configurationPath);
         }
     }
 }
