@@ -16,19 +16,21 @@
 
 package org.gradle.tooling.model;
 
+import org.gradle.api.Incubating;
 import org.gradle.api.Nullable;
+import org.gradle.tooling.model.gradle.GradleScript;
+
+import java.io.File;
 
 /**
- * Gradle project.
+ * Represents a Gradle project.
  *
  * @since 1.0-milestone-5
  */
 public interface GradleProject extends HierarchicalElement, BuildableElement {
 
     /**
-     * Returns the tasks of this project.
-     *
-     * @return The tasks.
+     * {@inheritDoc}
      */
     DomainObjectSet<? extends GradleTask> getTasks();
 
@@ -43,17 +45,45 @@ public interface GradleProject extends HierarchicalElement, BuildableElement {
     DomainObjectSet<? extends GradleProject> getChildren();
 
     /**
-     * Returns Gradle path.
+     * Returns the path of this project. This is a unique identifier for this project.
      *
      * @return The path.
      */
     String getPath();
 
     /**
-     * Searches all descendants (children, grand children, etc.), including self, by given path.
+     * Searches all descendants (children, grand-children, etc.), including self, by given path.
      *
      * @return Gradle project with matching path or {@code null} if not found.
      */
     @Nullable
     GradleProject findByPath(String path);
+
+    /**
+     * Returns the build script for this project.
+     *
+     * @return The build script.
+     * @since 1.8
+     */
+    @Incubating
+    GradleScript getBuildScript();
+
+    /**
+     * Returns the build directory for this project.
+     *
+     * @return The build directory.
+     * @since 2.0
+     */
+    @Incubating
+    File getBuildDirectory();
+
+    /**
+     * Returns the project directory for this project.
+     *
+     * @return The project directory.
+     * @since 2.4
+     */
+    @Incubating
+    File getProjectDirectory();
+
 }
